/**
 * @file GamePanel.java
 * @brief Définit la classe GamePanel pour le panneau de jeu principal.
 */

 package paquetGameUI;

 import paquetException.MonException;
 import paquetGameUI.*;
 
 import java.nio.file.Paths;
 import javax.swing.*;
 import java.awt.*;
 import java.awt.event.KeyEvent;
 import java.awt.event.KeyListener;
 import java.util.List;
 import java.io.IOException;
 
 /**
  * @class GamePanel
  * @brief La classe GamePanel représente le panneau de jeu principal.
  * 
  * La classe GamePanel gère le rendu graphique, les interactions avec le clavier et la navigation entre les niveaux.
  */
 public class GamePanel extends JPanel implements KeyListener {
     private List<Room> rooms; /**< Liste des salles du niveau. */
     private int totalHeight; /**< Hauteur totale des salles du niveau. */
     private int totalWidth; /**< Largeur totale des salles du niveau. */
     private int TILE_SIZE; /**< Taille des tuiles pour le rendu graphique. */
     private Player player; /**< Le joueur dans le jeu. */
     private int currentRoomIndex; /**< Index de la salle actuelle. */
     private int currentLevel; /**< Niveau actuel du jeu. */
     private final int maxLevel = 4; /**< Nombre maximum de niveaux dans le jeu. */
     private final MenuPanel menuPanel; /**< Panneau de menu associé à ce panneau de jeu. */
     private JButton nextButton; /**< Bouton pour passer au niveau suivant. */
     private JButton menuButton; /**< Bouton pour revenir au menu. */
     private boolean levelCompleted; /**< Indique si le niveau actuel est complété. */
     private Image heartImage; /**< Image du cœur pour représenter les vies du joueur. */
 
     /**
      * @brief Constructeur de la classe GamePanel.
      * 
      * @param menuPanel Le panneau de menu associé à ce panneau de jeu.
      * @throws MonException Si les images ne peuvent pas être chargées.
      */
     public GamePanel(MenuPanel menuPanel) throws MonException {
         this.menuPanel = menuPanel;
         currentLevel = 1;
         try {
             loadLevel(currentLevel);
             loadHeartImage(); // Charger l'image du cœur
         } catch (IOException e) {
             throw new MonException("Erreur lors du chargement du jeu", e);
         }
         setFocusable(true);
         requestFocusInWindow();
         addKeyListener(this);
         setupButtons();
     }
 
     /**
      * @brief Charge l'image du cœur.
      * 
      * @throws IOException Si l'image du cœur ne peut pas être chargée.
      */
     private void loadHeartImage() throws IOException {
         heartImage = new ImageIcon("paquetGameUI/images/heart.png").getImage(); // Assurez-vous que le chemin de l'image est correct
         if (heartImage == null) {
             throw new IOException("Erreur de chargement de l'image du cœur.");
         }
     }
 
     /**
      * @brief Configure les boutons de navigation (Menu et Suivant).
      */
     private void setupButtons() {
         setLayout(null);
 
         menuButton = new JButton("Menu");
         nextButton = new JButton("Suivant");
 
         add(menuButton);
         add(nextButton);
 
         menuButton.addActionListener(e -> returnToMenu());
         nextButton.addActionListener(e -> {
             try {
                 goToNextLevel();
             } catch (IOException e1) {
                 e1.printStackTrace();
             }
         });
 
         updateButtons();
     }
 
     /**
      * @brief Met à jour l'état des boutons de navigation.
      */
     private void updateButtons() {
         nextButton.setEnabled(levelCompleted && currentLevel < menuPanel.getUnlockedLevels());
     }
 
     /**
      * @brief Met à jour la position des boutons en bas de l'interface graphique.
      */
     private void updateButtonPositions() {
         int buttonWidth = 100;
         int buttonHeight = 30;
         int gap = 20; // Espace entre les boutons et l'affichage du niveau
 
         int buttonsY = 60 + totalHeight * TILE_SIZE + gap;
         int buttonsX = (getWidth() - 2 * buttonWidth - gap) / 2;
 
         menuButton.setBounds(buttonsX, buttonsY, buttonWidth, buttonHeight);
         nextButton.setBounds(buttonsX + buttonWidth + gap, buttonsY, buttonWidth, buttonHeight);
     }
 
     /**
      * @brief Retourne au menu principal.
      */
     private void returnToMenu() {
         JFrame frame = (JFrame) SwingUtilities.getWindowAncestor(this);
         frame.getContentPane().removeAll();
         frame.getContentPane().add(menuPanel);
         frame.revalidate();
         frame.repaint();
     }
 
     /**
      * @brief Passe au niveau suivant si le niveau actuel est complété.
      * @throws IOException Si une erreur survient lors du chargement du niveau suivant.
      */
     private void goToNextLevel() throws IOException {
         if (levelCompleted && currentLevel < menuPanel.getUnlockedLevels()) {
             currentLevel++;
             try {
                 loadLevel(currentLevel);
             } catch (MonException e) {
                 JOptionPane.showMessageDialog(this, "Erreur de chargement du niveau: " + e.getMessage());
             }
             repaint();
             updateButtons();
         } else {
             JOptionPane.showMessageDialog(this, "Ce niveau est verrouillé ou n'a pas été complété.");
         }
     }
 
     /**
      * @brief Charge un niveau spécifique.
      * 
      * @param level Le numéro du niveau à charger.
      * @throws MonException Si les fichiers de niveau ne peuvent pas être trouvés ou lus.
      * @throws IOException Si une erreur survient lors du chargement du niveau.
      */
     public void loadLevel(int level) throws MonException, IOException {
         currentLevel = level;
         levelCompleted = false;
         currentRoomIndex = 0; // Réinitialiser à la première salle du niveau
         String baseDir = "levels/";
         String[] roomFiles = {
             Paths.get(baseDir, "level" + level + "_room1.txt").toString(),
             Paths.get(baseDir, "level" + level + "_room2.txt").toString()
         };
         rooms = RoomLoader.loadRooms(roomFiles);
         if (rooms == null || rooms.isEmpty()) {
             throw new MonException("Les fichiers de niveau sont vides ou non valides.");
         }
         totalHeight = calculateTotalHeight();
         totalWidth = rooms.get(0).getLayout()[0].length;
         initializePlayer();
     }
 
     /**
      * @brief Initialise le joueur à la position de départ du niveau actuel.
      */
     private void initializePlayer() {
         int[][] layout = rooms.get(currentRoomIndex).getLayout();
         for (int y = 0; y < layout.length; y++) {
             for (int x = 0; x < layout[y].length; x++) {
                 if (layout[y][x] == 3 || layout[y][x] == 2) {
                     if (player == null) {
                         player = new Player(x, y);
                     } else {
                         player.setPosition(x, y);
                     }
                     return;
                 }
             }
         }
         rooms.get(currentRoomIndex).setPlayer(player); // Placer le joueur dans la salle actuelle
     }
 
     /**
      * @brief Calcule la hauteur totale des salles du niveau.
      * 
      * @return La hauteur totale.
      */
     private int calculateTotalHeight() {
         int height = 0;
         for (Room room : rooms) {
             height += room.getHeight();
         }
         return height;
     }
 
     /**
      * @brief Redéfinit la méthode paintComponent pour dessiner le panneau de jeu.
      * 
      * @param g L'objet Graphics utilisé pour dessiner.
      */
     protected void paintComponent(Graphics g) {
         super.paintComponent(g);
 
         TILE_SIZE = Math.min((getWidth() / totalWidth), (getHeight() - 100) / totalHeight);
 
         int offsetY = 60;
         int offsetX = (getWidth() - totalWidth * TILE_SIZE) / 2;
 
         for (int i = 0; i < rooms.size(); i++) {
             rooms.get(i).draw(g, offsetX, offsetY, TILE_SIZE);
             if (i == currentRoomIndex) {
                 g.drawImage(player.getPlayerImage(), player.getX() * TILE_SIZE + offsetX, player.getY() * TILE_SIZE + offsetY, TILE_SIZE, TILE_SIZE, null);
             }
             offsetY += rooms.get(i).getHeight() * TILE_SIZE;
         }
         drawHUD(g);
 
         updateButtonPositions();
     }
 
     /**
      * @brief Dessine l'interface utilisateur (HUD) montrant les vies et les objets collectés.
      * 
      * @param g L'objet Graphics utilisé pour dessiner.
      */
     private void drawHUD(Graphics g) {
         int heartSize = 20; // Taille des cœurs
         int x = 20;
         int y = 5;
         
         for (int i = 0; i < player.getLives(); i++) {
             g.drawImage(heartImage, x, y, heartSize, heartSize, this);
             x += heartSize + 5;
         }
         g.setColor(Color.BLACK);
         g.drawString("Collecte: " + player.getItemsCollected(), getWidth() - 100, 20);
         String levelText = "Niveau: " + currentLevel;
         FontMetrics fm = g.getFontMetrics();
         int levelX = (getWidth() - fm.stringWidth(levelText)) / 2;
         g.drawString(levelText, levelX, 50);
     }
 
     /**
      * @brief Retourne les dimensions préférées du panneau.
      * 
      * @return La dimension préférée.
      */
     public Dimension getPreferredSize() {
         return new Dimension(800, 600);
     }
 
     /**
      * @brief Gère les événements de touche enfoncée.
      * 
      * @param e L'événement de touche.
      */
     public void keyPressed(KeyEvent e) {
         Room currentRoom = rooms.get(currentRoomIndex);
         int[][] layout = rooms.get(currentRoomIndex).getLayout();
         player.keyPressed(e, currentRoom, currentRoomIndex, currentLevel);
 
         if (player.isOnDoor(layout)) {
             if (player.hasRoomKey(currentRoomIndex)) {
                 currentRoomIndex = (currentRoomIndex + 1) % rooms.size();
                 initializePlayer();
                 layout = rooms.get(currentRoomIndex).getLayout();
             } else {
                 JOptionPane.showMessageDialog(this, "Pour explorer une nouvelle salle, veuillez récupérer sa clé.");
             }
         }
 
         if (player.isOnExit(layout)) {
             if (currentRoomIndex == rooms.size() - 1) {
                 if (currentLevel == maxLevel) {
                     JOptionPane.showMessageDialog(this, "VOTRE MISSION EST TERMINÉE CHAMPION");
                     System.exit(0);
                 } else {
                     if (player.hasLevelKey(currentLevel)) {
                         levelCompleted = true;
                         updateButtons();
                         menuPanel.unlockNextLevel(currentLevel);
                         JOptionPane.showMessageDialog(this, "Niveau complété! Vous pouvez passer au niveau suivant.");
                     } else {
                         JOptionPane.showMessageDialog(this, "Pour passer au niveau suivant, veuillez récupérer la clé de niveau.");
                     }
                 }
             } else {
                 if (player.hasRoomKey(currentRoomIndex)) {
                     currentRoomIndex++;
                     initializePlayer();
                 } else {
                     JOptionPane.showMessageDialog(this, "Pour explorer une nouvelle salle, veuillez récupérer sa clé.");
                 }
             }
         }
 
         if (player.isGameOver()) {
             JOptionPane.showMessageDialog(this, "Game Over! Vous avez perdu toutes vos vies.");
             System.exit(0);
         }
 
         repaint();
     }
 
     /**
      * @brief Gère les événements de touche relâchée.
      * 
      * @param e L'événement de touche.
      */
     public void keyReleased(KeyEvent e) {}
 
     /**
      * @brief Gère les événements de touche tapée.
      * 
      * @param e L'événement de touche.
      */
     public void keyTyped(KeyEvent e) {}
 }
 