package paquetGameUI;

import paquetException.MonException;
import paquetGameUI.GamePanel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;

/**
 * Classe représentant le panneau de menu principal du jeu.
 */
public class MenuPanel extends JPanel {
    private final ImageIcon lockIcon;
    private final ImageIcon unlockIcon;
    private final int maxLevel = 4;
    private final int iconSize = 50; // Taille réduite des icônes
    private int unlockedLevels = 1; // Le niveau initial déverrouillé

    /**
     * Constructeur de la classe MenuPanel.
     * Initialise les icônes et configure le panneau.
     */
    public MenuPanel() {
        lockIcon = new ImageIcon("paquetGameUI/images/lock.png"); // Assurez-vous d'avoir une image de cadenas
        unlockIcon = new ImageIcon("paquetGameUI/images/unlock.png"); // Assurez-vous d'avoir une image de cadenas déverrouillé
        setPreferredSize(new Dimension(800, 600));
        setFocusable(true);
        addMouseListener(new MouseAdapter() {
        
            /**
             * Gestionnaire d'événements pour les clics de souris.
             * @param e l'événement de souris.
             */
            public void mouseClicked(MouseEvent e) {
                try {
                    handleMouseClick(e.getX(), e.getY());
                } catch (IOException e1) {
                    e1.printStackTrace();
                } catch (MonException e1) {
                    e1.printStackTrace();
                }
            }
        });
    }

    /**
     * Gère les clics de souris pour déterminer si un niveau est sélectionné.
     * @param x la position x du clic de souris.
     * @param y la position y du clic de souris.
     * @throws IOException si une erreur d'entrée/sortie se produit.
     * @throws MonException si une exception personnalisée se produit.
     */
    private void handleMouseClick(int x, int y) throws IOException, MonException {
        int startX = (getWidth() - iconSize) / 2;
        int startY = 250;
        int spacingY = 100;

        for (int i = 1; i <= maxLevel; i++) {
            int iconY = startY + (i - 1) * spacingY;
            if (x >= startX && x <= startX + iconSize && y >= iconY && y <= iconY + iconSize) {
                if (i <= unlockedLevels) {
                    openGameFrame(i);
                } else {
                    JOptionPane.showMessageDialog(this, "Ce niveau est verrouillé. Veuillez compléter les niveaux précédents.");
                }
                break;
            }
        }
    }

    /**
     * Redessine le composant en affichant les niveaux et le texte d'accueil.
     * @param g l'objet Graphics pour dessiner le composant.
     */
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);

        g.setFont(new Font("Arial", Font.BOLD, 24));
        g.setColor(Color.BLACK);
        drawCenteredString(g, "Bienvenue!", getWidth(), 50);

        g.setFont(new Font("Arial", Font.BOLD, 20));
        drawCenteredString(g, "Présentation du jeu:", getWidth(), 100);

        g.setFont(new Font("Arial", Font.PLAIN, 14));
        drawCenteredString(g, "Ce jeu consiste à explorer des salles, collecter des objets,", getWidth(), 150);
        drawCenteredString(g, "éviter ou combattre des monstres, et progresser à travers différents niveaux.", getWidth(), 170);
        drawCenteredString(g, "Cliquez sur un niveau pour commencer.", getWidth(), 190);

        int startX = (getWidth() - iconSize) / 2; // Centrer horizontalement
        int startY = 250; // Point de départ Y pour l'affichage des niveaux
        int spacingY = 100; // Espace vertical entre les icônes

        for (int i = 1; i <= maxLevel; i++) {
            ImageIcon icon = (i <= unlockedLevels) ? unlockIcon : lockIcon;
            int y = startY + (i - 1) * spacingY;
            g.drawImage(icon.getImage(), startX, y, iconSize, iconSize, this);
            drawCenteredString(g, "Niveau " + i, getWidth(), y + iconSize + 20);
        }
    }

    /**
     * Dessine une chaîne de caractères centrée horizontalement.
     * @param g l'objet Graphics pour dessiner le texte.
     * @param text le texte à dessiner.
     * @param width la largeur du composant.
     * @param y la position y où le texte doit être dessiné.
     */
    private void drawCenteredString(Graphics g, String text, int width, int y) {
        FontMetrics fm = g.getFontMetrics();
        int x = (width - fm.stringWidth(text)) / 2;
        g.drawString(text, x, y);
    }

    /**
     * Ouvre une nouvelle fenêtre de jeu pour le niveau spécifié.
     * @param level le niveau de jeu à charger.
     * @throws IOException si une erreur d'entrée/sortie se produit.
     * @throws MonException si une exception personnalisée se produit.
     */
    private void openGameFrame(int level) throws IOException, MonException {
        JFrame gameFrame = new JFrame("Game Level");
        GamePanel gamePanel = new GamePanel(this);
        gamePanel.loadLevel(level);
        gameFrame.add(gamePanel);
        gameFrame.pack();
        gameFrame.setSize(800, 600);
        gameFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        gameFrame.setVisible(true);
    }

    /**
     * Déverrouille le niveau suivant si le niveau actuel est complété.
     * @param currentLevel le niveau actuel complété.
     */
    public void unlockNextLevel(int currentLevel) {
        if (currentLevel == unlockedLevels && unlockedLevels < maxLevel) {
            unlockedLevels++;
            repaint();
        }
    }

    /**
     * Retourne le nombre de niveaux déverrouillés.
     * @return le nombre de niveaux déverrouillés.
     */
    public int getUnlockedLevels() {
        return unlockedLevels;
    }

    /**
     * Point d'entrée principal de l'application.
     * @param args les arguments de la ligne de commande.
     */
    public static void main(String[] args) {
        JFrame frame = new JFrame("Menu du Jeu");
        MenuPanel menuPanel = new MenuPanel();
        frame.add(menuPanel);
        frame.pack();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setVisible(true);
    }
}
