package paquetGameUI;

import java.awt.Image;
import java.awt.event.KeyEvent;
import javax.swing.ImageIcon;
import java.util.HashSet;
import java.util.Set;

public class Player {
    public static final int UP = 0;
    public static final int DOWN = 1;
    public static final int LEFT = 2;
    public static final int RIGHT = 3;

    private int x, y;
    private final Image playerImage;
    private int lives;
    private int itemsCollected;
    private Set<Integer> roomKeysCollected;
    private Set<Integer> levelKeysCollected;

    /**
     * Constructeur de la classe Player.
     * 
     * @param startX Position initiale en X.
     * @param startY Position initiale en Y.
     */
    public Player(int startX, int startY) {
        this.x = startX;
        this.y = startY;
        this.playerImage = new ImageIcon("paquetGameUI/images/player.png").getImage();
        this.lives = 3;
        this.itemsCollected = 0;
        this.roomKeysCollected = new HashSet<>();
        this.levelKeysCollected = new HashSet<>();
    }

    /**
     * Dépose une bombe dans la salle donnée dans la direction spécifiée.
     * 
     * @param room      La salle où la bombe doit être déposée.
     * @param direction La direction dans laquelle déposer la bombe.
     */
    public void dropBomb(Room room, int direction) {
        int bombX = x;
        int bombY = y;

        switch (direction) {
            case UP:
                bombY -= 1;
                break;
            case DOWN:
                bombY += 1;
                break;
            case LEFT:
                bombX -= 1;
                break;
            case RIGHT:
                bombX += 1;
                break;
        }

        // Vérifier que la position de la bombe est valide (dans les limites de la salle)
        int[][] layout = room.getLayout();
        if (bombX >= 0 && bombX < layout[0].length && bombY >= 0 && bombY < layout.length && layout[bombY][bombX] != 1) {
            room.addBomb(bombX, bombY);
        }
    }

    /**
     * Gère les événements de touche en fonction de la salle actuelle, l'index de la salle et le niveau actuel.
     * 
     * @param e               L'événement de touche.
     * @param room            La salle actuelle.
     * @param currentRoomIndex L'index de la salle actuelle.
     * @param currentLevel     Le niveau actuel.
     */
    public void keyPressed(KeyEvent e, Room room, int currentRoomIndex, int currentLevel) {
        int key = e.getKeyCode();
        int[][] layout = room.getLayout();

        switch (key) {
            case KeyEvent.VK_LEFT:
                move(-1, 0, layout, currentRoomIndex, currentLevel);
                break;
            case KeyEvent.VK_RIGHT:
                move(1, 0, layout, currentRoomIndex, currentLevel);
                break;
            case KeyEvent.VK_UP:
                move(0, -1, layout, currentRoomIndex, currentLevel);
                break;
            case KeyEvent.VK_DOWN:
                move(0, 1, layout, currentRoomIndex, currentLevel);
                break;
            case KeyEvent.VK_NUMPAD4:
                dropBomb(room, LEFT);
                break;
            case KeyEvent.VK_NUMPAD6:
                dropBomb(room, RIGHT);
                break;
            case KeyEvent.VK_NUMPAD8:
                dropBomb(room, UP);
                break;
            case KeyEvent.VK_NUMPAD2:
                dropBomb(room, DOWN);
                break;
            case KeyEvent.VK_S:
                dropBomb(room, LEFT);
                break;
            case KeyEvent.VK_F:
                dropBomb(room, RIGHT);
                break;
            case KeyEvent.VK_E:
                dropBomb(room, UP);
                break;
            case KeyEvent.VK_D:
                dropBomb(room, DOWN);
                break;
        }
    }

    /**
     * Vérifie si le joueur est sur une porte.
     * 
     * @param layout La disposition de la salle.
     * @return true si le joueur est sur une porte, false sinon.
     */
    public boolean isOnDoor(int[][] layout) {
        return layout[y][x] == 5;
    }

    /**
     * Vérifie si le joueur est sur une sortie.
     * 
     * @param layout La disposition de la salle.
     * @return true si le joueur est sur une sortie, false sinon.
     */
    public boolean isOnExit(int[][] layout) {
        return layout[y][x] == 4;
    }

    /**
     * Obtient la position en X du joueur.
     * 
     * @return La position en X.
     */
    public int getX() {
        return x;
    }

    /**
     * Obtient la position en Y du joueur.
     * 
     * @return La position en Y.
     */
    public int getY() {
        return y;
    }

    /**
     * Définit la position du joueur.
     * 
     * @param x La nouvelle position en X.
     * @param y La nouvelle position en Y.
     */
    public void setPosition(int x, int y) {
        this.x = x;
        this.y = y;
    }

    /**
     * Obtient l'image du joueur.
     * 
     * @return L'image du joueur.
     */
    public Image getPlayerImage() {
        return playerImage;
    }

    /**
     * Déplace le joueur dans la direction spécifiée si le déplacement est valide.
     * 
     * @param dx              Le déplacement en X.
     * @param dy              Le déplacement en Y.
     * @param layout          La disposition de la salle.
     * @param currentRoomIndex L'index de la salle actuelle.
     * @param currentLevel     Le niveau actuel.
     */
    public void move(int dx, int dy, int[][] layout, int currentRoomIndex, int currentLevel) {
        int newX = x + dx;
        int newY = y + dy;

        if (newX >= 0 && newX < layout[0].length && newY >= 0 && newY < layout.length && layout[newY][newX] != 1) {
            x = newX;
            y = newY;
            if (layout[newY][newX] == 6 || layout[newY][newX] == 9 || layout[newY][newX] == 10 || layout[newY][newX] == 11) {
                itemsCollected++;
                layout[newY][newX] = 0;
            }
            if (layout[newY][newX] == 7) {
                roomKeysCollected.add(currentRoomIndex);
                layout[newY][newX] = 0;
            }
            if (layout[newY][newX] == 8) {
                levelKeysCollected.add(currentLevel);
                layout[newY][newX] = 0;
            }
            if (layout[newY][newX] >= 12 && layout[newY][newX] <= 15) {
                loseLife();
                layout[newY][newX] = 0;
            }
        }
    }

    /**
     * Réduit le nombre de vies du joueur de 1.
     */
    public void loseLife() {
        lives--;
    }

    /**
     * Vérifie si le jeu est terminé (si le joueur n'a plus de vies).
     * 
     * @return true si le joueur n'a plus de vies, false sinon.
     */
    public boolean isGameOver() {
        return lives <= 0;
    }

    /**
     * Obtient le nombre de vies restantes du joueur.
     * 
     * @return Le nombre de vies.
     */
    public int getLives() {
        return lives;
    }

    /**
     * Obtient le nombre d'objets collectés par le joueur.
     * 
     * @return Le nombre d'objets collectés.
     */
    public int getItemsCollected() {
        return itemsCollected;
    }

    /**
     * Vérifie si le joueur a une clé de salle spécifique.
     * 
     * @param roomIndex L'index de la salle.
     * @return true si le joueur a la clé de la salle, false sinon.
     */
    public boolean hasRoomKey(int roomIndex) {
        return roomKeysCollected.contains(roomIndex);
    }

    /**
     * Vérifie si le joueur a une clé de niveau spécifique.
     * 
     * @param levelIndex L'index du niveau.
     * @return true si le joueur a la clé du niveau, false sinon.
     */
    public boolean hasLevelKey(int levelIndex) {
        return levelKeysCollected.contains(levelIndex);
    }

    /**
     * Obtient la direction actuelle du joueur.
     * 
     * @return La direction actuelle.
     * @throws UnsupportedOperationException Si la méthode n'est pas implémentée.
     */
    public int getDirection() {
        throw new UnsupportedOperationException("Unimplemented method 'getDirection'");
    }

    /**
     * Collecte une clé de salle.
     * 
     * @param roomIndex L'index de la salle.
     */
    public void collectRoomKey(int roomIndex) {
        roomKeysCollected.add(roomIndex);
    }

    /**
     * Collecte une clé de niveau.
     * 
     * @param levelIndex L'index du niveau.
     */
    public void collectLevelKey(int levelIndex) {
        levelKeysCollected.add(levelIndex);
    }
}
