package paquetGameUI;

// import java.nio.file.Paths;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import javax.swing.ImageIcon;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import javax.swing.Timer;

public class Room {
    private final int[][] layout;
    private final Image diamondImage;
    private final Image treasureImage;
    private final Image crownImage;
    private final Image roomKeyImage;
    private final Image levelKeyImage;
    private final Image ringImage;
    private final Image monster1Image;
    private final Image monster2Image;
    private final Image monster3Image;
    private final Image monster4Image;
    private final List<Bomb> bombs;
    private Player player;
    private Random random;

    /**
     * Constructeur de la classe Room.
     * Initialise la disposition de la pièce et charge les images des éléments.
     * @param layout Disposition initiale de la pièce
     */
    public Room(int[][] layout) {
        this.layout = layout;
        this.diamondImage = new ImageIcon("paquetGameUI/images/diamond.jpg").getImage();
        this.treasureImage = new ImageIcon("paquetGameUI/images/treasure.png").getImage();
        this.crownImage = new ImageIcon("paquetGameUI/images/crown.png").getImage();
        this.roomKeyImage = new ImageIcon("paquetGameUI/images/room_key.png").getImage();
        this.levelKeyImage = new ImageIcon("paquetGameUI/images/level_key.jpg").getImage();
        this.ringImage = new ImageIcon("paquetGameUI/images/ring.png").getImage();
        this.monster1Image = new ImageIcon("paquetGameUI/images/koro.jpg").getImage();
        this.monster2Image = new ImageIcon("paquetGameUI/images/natsu.jpg").getImage();
        this.monster3Image = new ImageIcon("paquetGameUI/images/monstre.jpg").getImage();
        this.monster4Image = new ImageIcon("paquetGameUI/images/serpent.jpg").getImage();
        this.bombs = new ArrayList<>();
        this.random = new Random();

        // Timer pour le déplacement des monstres
        new Timer(1000, e -> moveMonsters()).start();
    }

    /**
     * Définit le joueur dans la pièce.
     * @param player Le joueur à ajouter à la pièce
     */
    public void setPlayer(Player player) {
        this.player = player;
    }

    /**
     * Ajoute une bombe à la position spécifiée.
     * @param x Position x de la bombe
     * @param y Position y de la bombe
     */
    public void addBomb(int x, int y) {
        Bomb bomb = new Bomb(x, y);
        bombs.add(bomb);

        // Configure un Timer pour vérifier les collisions de manière répétée
        new Timer(100, e -> {
            if (checkCollisions(bomb)) {
                bombs.remove(bomb);
                ((Timer) e.getSource()).stop();
            }
        }).start();
    }

    /**
     * Vérifie les collisions entre une bombe et les éléments environnants.
     * @param bomb La bombe pour laquelle vérifier les collisions
     * @return true si une collision est détectée, sinon false
     */
    private boolean checkCollisions(Bomb bomb) {
        int bombX = bomb.getX();
        int bombY = bomb.getY();
        int[][] layout = getLayout();
        boolean collisionDetected = false;

        for (int y = Math.max(0, bombY - 1); y <= Math.min(layout.length - 1, bombY + 1); y++) {
            for (int x = Math.max(0, bombX - 1); x <= Math.min(layout[0].length - 1, bombX + 1); x++) {
                // Vérifie s'il y a un monstre
                if (layout[y][x] >= 12 && layout[y][x] <= 15) {
                    layout[y][x] = 0; // Supprime le monstre
                    collisionDetected = true;
                }
                // Vérifie si le joueur est dans la zone d'explosion
                if (player != null && player.getX() == x && player.getY() == y) {
                    player.loseLife();
                    collisionDetected = true;
                }
            }
        }

        return collisionDetected;
    }

    /**
     * Déplace les monstres dans la pièce.
     */
    private void moveMonsters() {
        int[][] layout = getLayout();

        for (int y = 0; y < layout.length; y++) {
            for (int x = 0; x < layout[y].length; x++) {
                if (layout[y][x] >= 12 && layout[y][x] <= 15) {
                    int monsterType = layout[y][x];
                    int direction = random.nextInt(4);
                    int newX = x, newY = y;

                    switch (direction) {
                        case 0: newY -= 1; break; // Haut
                        case 1: newY += 1; break; // Bas
                        case 2: newX -= 1; break; // Gauche
                        case 3: newX += 1; break; // Droite
                    }

                    // Vérifie si la nouvelle position est dans les limites et n'est pas un mur ou un autre monstre
                    if (newX >= 0 && newX < layout[0].length && newY >= 0 && newY < layout.length &&
                        layout[newY][newX] == 0) {
                        layout[y][x] = 0; // Efface l'ancienne position
                        layout[newY][newX] = monsterType; // Déplace le monstre à la nouvelle position
                    }
                }
            }
        }
    }

    /**
     * Dessine la pièce et ses éléments.
     * @param g L'objet Graphics utilisé pour dessiner
     * @param offsetX Décalage en x pour le dessin
     * @param offsetY Décalage en y pour le dessin
     * @param tileSize Taille de chaque tuile
     */
    public void draw(Graphics g, int offsetX, int offsetY, int tileSize) {
        for (int y = 0; y < layout.length; y++) {
            for (int x = 0; x < layout[y].length; x++) {
                int tile = layout[y][x];
                switch (tile) {
                    case 0:
                        g.setColor(Color.WHITE);
                        g.fillRect(x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize);
                        break;
                    case 1:
                        g.setColor(Color.BLACK);
                        g.fillRect(x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize);
                        break;
                    case 2:
                        g.setColor(Color.YELLOW);
                        g.fillRect(x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize);
                        break;
                    case 3:
                        g.setColor(Color.WHITE); // Fond pour l'image du joueur
                        g.fillRect(x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize);
                        break;
                    case 4:
                        g.setColor(Color.GREEN);
                        g.fillRect(x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize);
                        break;
                    case 5:
                        g.setColor(Color.ORANGE); // Couleur pour la porte
                        g.fillRect(x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize);
                        break;
                    case 6:
                        g.drawImage(diamondImage, x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize, null);
                        break;
                    case 7:
                        g.drawImage(roomKeyImage, x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize, null);
                        break;
                    case 8:
                        g.drawImage(levelKeyImage, x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize, null);
                        break;
                    case 9:
                        g.drawImage(treasureImage, x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize, null);
                        break;
                    case 10:
                        g.drawImage(crownImage, x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize, null);
                        break;
                    case 11:
                        g.drawImage(ringImage, x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize, null);
                        break;
                    case 12:
                        g.drawImage(monster1Image, x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize, null);
                        break;
                    case 13:
                        g.drawImage(monster2Image, x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize, null);
                        break;
                    case 14:
                        g.drawImage(monster3Image, x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize, null);
                        break;
                    case 15:
                        g.drawImage(monster4Image, x * tileSize + offsetX, y * tileSize + offsetY, tileSize, tileSize, null);
                        break;
                }
            }
        }

        // Dessine les bombes
        for (Bomb bomb : bombs) {
            bomb.draw(g, tileSize, offsetX, offsetY);
        }
    }

    /**
     * Obtient la disposition de la pièce.
     * @return Un tableau 2D représentant la disposition de la pièce
     */
    public int[][] getLayout() {
        return layout;
    }

    /**
     * Obtient la hauteur de la pièce.
     * @return La hauteur de la pièce
     */
    public int getHeight() {
        return layout.length;
    }
}
