package paquetTerminal;

import paquetException.MonException;
import paquetGameUI.Player;
import paquetGameUI.Room;
import paquetGameUI.RoomLoader;

import java.io.IOException;
import java.nio.file.Paths;
import java.util.List;
import java.util.Scanner;

/**
 * Classe représentant le jeu dans le terminal.
 */
public class TerminalGame {
    private List<Room> rooms;
    private Player player;
    private int currentRoomIndex;
    private int currentLevel;
    private final int maxLevel = 4;
    private boolean levelCompleted;
    private int totalHeight;
    private int totalWidth;

    /**
     * Constructeur de la classe TerminalGame.
     * @throws MonException Si une erreur spécifique se produit.
     * @throws IOException Si une erreur d'entrée/sortie se produit.
     */
    public TerminalGame() throws MonException, IOException {
        currentLevel = 1;
        loadLevel(currentLevel);
    }

    /**
     * Calcule la hauteur totale des salles du niveau.
     * 
     * @return La hauteur totale.
     */
    private int calculateTotalHeight() {
        int height = 0;
        for (Room room : rooms) {
            height += room.getHeight();
        }
        return height;
    }

    /**
     * Charge un niveau spécifique du jeu.
     * 
     * @param level Le niveau à charger.
     * @throws MonException Si une erreur spécifique se produit.
     * @throws IOException Si une erreur d'entrée/sortie se produit.
     */
    public void loadLevel(int level) throws MonException, IOException {
        currentLevel = level;
        levelCompleted = false;
        currentRoomIndex = 0; // Réinitialiser à la première salle du niveau
        String baseDir = "levels/";
        String[] roomFiles = {
            Paths.get(baseDir, "level" + level + "_room1.txt").toString(),
            Paths.get(baseDir, "level" + level + "_room2.txt").toString()
        };
        rooms = RoomLoader.loadRooms(roomFiles);
        if (rooms == null || rooms.isEmpty()) {
            throw new MonException("Les fichiers de niveau sont vides ou non valides.");
        }
        totalHeight = calculateTotalHeight();
        totalWidth = rooms.get(0).getLayout()[0].length;
        initializePlayer();
    }

    /**
     * Initialise le joueur dans la salle actuelle.
     */
    private void initializePlayer() {
        int[][] layout = rooms.get(currentRoomIndex).getLayout();
        for (int y = 0; y < layout.length; y++) {
            for (int x = 0; x < layout[y].length; x++) {
                if (layout[y][x] == 3 || layout[y][x] == 2) {
                    if (player == null) {
                        player = new Player(x, y);
                    } else {
                        player.setPosition(x, y);
                    }
                    return;
                }
            }
        }
        rooms.get(currentRoomIndex).setPlayer(player); // Placer le joueur dans la salle actuelle
    }

    /**
     * Démarre le jeu dans le terminal.
     * @throws IOException Si une erreur d'entrée/sortie se produit.
     */
    public void start() throws IOException {
        Scanner scanner = new Scanner(System.in);
        while (true) {
            printCurrentRooms();
            System.out.println("Utilisez les touches H, B, G, D pour vous déplacer, ");
            System.out.println("2, 4, 8, 6 pour jeter des bombes dans les directions bas, gauche, haut, droite, ");
            System.out.println("et Q pour quitter.");
            String command = scanner.nextLine();
            if (command.equalsIgnoreCase("Q")) {
                System.out.println("Merci d'avoir joué !");
                break;
            }
            handleCommand(command);
        }
        scanner.close();
    }

    /**
     * Affiche les salles actuelles du niveau.
     */
    private void printCurrentRooms() {
        clearScreen();
        Room room1 = rooms.get(0);
        Room room2 = rooms.get(1);

        int[][] layout1 = room1.getLayout();
        int[][] layout2 = room2.getLayout();

        int roomHeight = Math.max(layout1.length, layout2.length);
        int roomWidth1 = layout1[0].length;
        int roomWidth2 = layout2[0].length;

        for (int y = 0; y < roomHeight; y++) {
            for (int x = 0; x < roomWidth1; x++) {
                if (y < layout1.length) {
                    if (player.getX() == x && player.getY() == y && currentRoomIndex == 0) {
                        System.out.print("P ");
                    } else {
                        printCell(layout1[y][x]);
                    }
                } else {
                    System.out.print("  ");
                }
            }
            System.out.print("   "); // Séparation entre les deux salles
            for (int x = 0; x < roomWidth2; x++) {
                if (y < layout2.length) {
                    if (player.getX() == x && player.getY() == y && currentRoomIndex == 1) {
                        System.out.print("P ");
                    } else {
                        printCell(layout2[y][x]);
                    }
                } else {
                    System.out.print("  ");
                }
            }
            System.out.println();
        }
    }

    /**
     * Affiche la cellule donnée.
     * 
     * @param cellValue La valeur de la cellule à afficher.
     */
    private void printCell(int cellValue) {
        switch (cellValue) {
            case 0:
                System.out.print(". ");
                break;
            case 1:
                System.out.print("# ");
                break;
            case 2:
                System.out.print("E ");
                break;
            case 3:
                System.out.print("S ");
                break;
            case 4:
                System.out.print("X ");
                break;
            case 5:
                System.out.print("D ");
                break;
            case 7:
                System.out.print("K ");
                break;
            case 8:
                System.out.print("L ");
                break;
            case 9:
                System.out.print("T ");
                break;
            case 10:
                System.out.print("C ");
                break;
            case 11:
                System.out.print("B ");
                break;
            case 12:
                System.out.print("M ");
                break;
            case 13:
                System.out.print("* ");
                break;
            default:
                System.out.print("? ");
                break;
        }
    }

    /**
     * Efface l'écran pour un affichage propre.
     */
    private void clearScreen() {
        System.out.print("\033[H\033[2J");
        System.out.flush();
    }

    /**
     * Gère la commande entrée par l'utilisateur.
     * 
     * @param command La commande entrée par l'utilisateur.
     * @throws IOException Si une erreur d'entrée/sortie se produit.
     */
    private void handleCommand(String command) throws IOException {
        switch (command.toUpperCase()) {
            case "H":
                player.move(0, -1, rooms.get(currentRoomIndex).getLayout(), currentRoomIndex, currentLevel);
                break;
            case "G":
                player.move(-1, 0, rooms.get(currentRoomIndex).getLayout(), currentRoomIndex, currentLevel);
                break;
            case "B":
                player.move(0, 1, rooms.get(currentRoomIndex).getLayout(), currentRoomIndex, currentLevel);
                break;
            case "D":
                player.move(1, 0, rooms.get(currentRoomIndex).getLayout(), currentRoomIndex, currentLevel);
                break;
            case "4":
                player.dropBomb(rooms.get(currentRoomIndex), Player.LEFT);
                break;
            case "6":
                player.dropBomb(rooms.get(currentRoomIndex), Player.RIGHT);
                break;
            case "8":
                player.dropBomb(rooms.get(currentRoomIndex), Player.UP);
                break;
            case "2":
                player.dropBomb(rooms.get(currentRoomIndex), Player.DOWN);
                break;
            default:
                System.out.println("Commande inconnue.");
                break;
        }
        checkPlayerStatus();
    }

    /**
     * Vérifie l'état du joueur et met à jour le jeu en conséquence.
     * 
     * @throws IOException Si une erreur d'entrée/sortie se produit.
     */
    private void checkPlayerStatus() throws IOException {
        int[][] layout = rooms.get(currentRoomIndex).getLayout();
        if (player.isOnDoor(layout)) {
            if (player.hasRoomKey(currentRoomIndex)) {
                currentRoomIndex = (currentRoomIndex + 1) % rooms.size();
                initializePlayer();
            } else {
                System.out.println("Pour explorer une nouvelle salle, veuillez récupérer sa clé.");
            }
        }

        if (player.isOnExit(layout)) {
            if (currentRoomIndex == rooms.size() - 1) {
                if (currentLevel == maxLevel) {
                    System.out.println("VOTRE MISSION EST TERMINÉE CHAMPION");
                    System.exit(0);
                } else {
                    if (player.hasLevelKey(currentLevel)) {
                        levelCompleted = true;
                        System.out.println("Niveau complété! Vous pouvez passer au niveau suivant.");
                        currentLevel++;
                        try {
                            loadLevel(currentLevel);
                        } catch (MonException e) {
                            System.out.println("Erreur de chargement du niveau: " + e.getMessage());
                        }
                    } else {
                        System.out.println("Pour passer au niveau suivant, veuillez récupérer la clé de niveau.");
                    }
                }
            } else {
                if (player.hasRoomKey(currentRoomIndex)) {
                    currentRoomIndex++;
                    initializePlayer();
                } else {
                    System.out.println("Pour explorer une nouvelle salle, veuillez récupérer sa clé.");
                }
            }
        }

        if (player.isGameOver()) {
            System.out.println("Game Over! Vous avez perdu toutes vos vies.");
            System.exit(0);
        }
    }

    /**
     * Point d'entrée du programme.
     * 
     * @param args Les arguments de la ligne de commande.
     * @throws IOException Si une erreur d'entrée/sortie se produit.
     */
    public static void main(String[] args) throws IOException {
        try {
            TerminalGame game = new TerminalGame();
            game.start();
        } catch (MonException e) {
            System.out.println("Erreur: " + e.getMessage());
        }
    }
}
